// Stepper: 0xC0 .. 0xFF

#include <p18cxxx.h>   /* for TRISB and PORTB declarations */
#include <usart.h>
#include <delays.h>
#include "config.h"





#pragma	romdata CONFIG=0x300000
unsigned char rom _CONFIG0 = _CP_OFF_0;
unsigned char rom _CONFIG1 = _OSCS_OFF_1 & _HS_OSC_1;
unsigned char rom _CONFIG2 = _BOR_ON_2 & _BORV_25_2 & _PWRT_ON_2;
unsigned char rom _CONFIG3 = _WDT_OFF_3;
unsigned char rom _CONFIG5 = _CCP2MX_ON_5;
unsigned char rom _CONFIG6 = _STVR_ON_6;
unsigned char rom _CONFIG7 = _LVP_OFF_7;
#pragma romdata



#define QUARZ 20000000
#define TAKT (QUARZ/4)


#define Delay_100us(x) Delay100TCYx(x*TAKT/1000000)
#define Delay_ms(x)     Delay1KTCYx(x*TAKT/1000000)


#define S1      PORTAbits.RA2 // S1
#define S2      PORTAbits.RA3 // S2
#define S3      PORTAbits.RA4 // S3
#define S4      PORTAbits.RA5 // S4

#define Y1      PORTBbits.RB0 // Y1
#define Y2      PORTBbits.RB1 // Y2
#define Y3      PORTBbits.RB2 // Y3
#define Y4      PORTBbits.RB3 // Y4
#define X1      PORTBbits.RB4 // X1
#define X2      PORTBbits.RB5 // X2
#define X3      PORTBbits.RB6 // X3
#define X4      PORTBbits.RB7 // X4

#define VSW     PORTCbits.RC0 // Schaltspannung
#define ENP     PORTCbits.RC1 // Enable Lautsprecher
#define CS  	PORTCbits.RC2 // CS
#define SCK		PORTCbits.RC3 // CLK
#define SDI		PORTCbits.RC4 // MCC OUT
#define SDO		PORTCbits.RC5 // MCC IN



#define P_LED3 PORTEbits.RE0 // LED3
#define P_LED2 PORTEbits.RE1 // LED2
#define P_LED1 PORTEbits.RE2 // LED1


#define TX_REF 0xFE

void high_irq(void);
void low_irq(void);
volatile unsigned dummyb;
unsigned char b;
unsigned char testi;
unsigned char buffer[64];
unsigned char wpointer;
unsigned char rpointer;
unsigned int adh,adl;
unsigned int block;
unsigned char taste;
unsigned char playspeed;
struct {
    unsigned play:1;
    unsigned rec:1;
} flags;



void putc(char c)
{
   while (TXSTAbits.TRMT==0);   
   TXREG=c;
}

void irq_disable_global(void)
{
   INTCON &= 0x7F;
}

void irq_enable_global(void)
{
   INTCON |= 0x80;
}

char SPI(char d)		// ber SPI-Schnittstelle senden
{				// 
SSPBUF=d;
while (!SSPSTATbits.BF);			// 	Warten bis gesendet ist
return SSPBUF;			// gleichzeitig empfangen
}



//**********************************************************************

char Command(char befF,unsigned int AdrH,unsigned int AdrL,char befH )
{				// Einen Befehl zum MMC senden
char a;
SPI(0xFF);
SPI(befF);
SPI((AdrH>>8)& 0xFF);
SPI(AdrH& 0xFF);
SPI((AdrL>>8)& 0xFF);
SPI(AdrL & 0xFF);
SPI(befH);
SPI(0xFF);
return SPI(0xFF);		// Response als Rckgabewert
}
//**************************************************************************

int MMC_Init()
{
int i;		// Variablen
// Init SPI
SSPSTATbits.SMP=0;		// Input ist gltig in der Mitte des Clock's
SSPSTATbits.CKE=0;		// Bei steigender Flanke Datenbernahme
SSPCON1bits.CKP=1;		// High ist passiver Zustand
SSPCON1bits.SSPM3=0;
SSPCON1bits.SSPM2=0;
SSPCON1bits.SSPM1=0;	// Geschwindigkeit f/64(312kHz), Master
SSPCON1bits.SSPM0=0;	// Geschwindigkeit f/16(1,25MHz), Master
SSPCON1bits.SSPEN=1;	// SPI Ein

CS=1;		// MMC-Disabled


//MMC in SPI-Modus starten, Reset
for(i=0; i < 10; i++)SPI(0xFF);		// 10*8=80 mal takten
CS=0;					// MMC-Enabled

// CMD0
if (Command(0x40,0,0,0x95) !=1)goto Fehler ; 	// Reset

st:						// Wenn MMC nicht da ist, bleibt
// CMD1						// Programm hier stehen
if (Command(0x41,0,0,0xFF) !=0) goto st ;	// CMD1 solange wiederholen

return 1;
Fehler:
return 0;
}








void irq_init(void)

{
   //IRQ-Priiority is activ
   RCON = 0b00000000;

   //Configuration for external IRQ 0, 1, PORTB.5
   //PWM-IN-Input
   INTCON  = 0b00100000;
   INTCON2 = 0b00000101;
   INTCON3 = 0b00000000;


}
#pragma code low_vector=0x18
void interrupt_at_low_vector(void)

{
   _asm GOTO high_irq _endasm


}
#pragma code /* return to the default code section */
#pragma interruptlow low_irq
void low_irq(void)

{
 testi=12;
}
#pragma code high_vector=0x08
void interrupt_at_high_vector(void)

{
   _asm GOTO high_irq _endasm


}
#pragma code /* return to the default code section */
#pragma interrupt high_irq
void high_irq (void)
{
unsigned char po_b;

   if (INTCONbits.RBIF == 1)
   {
	dummyb=PORTB;
	INTCONbits.RBIF=0;
   }


   //Timer0 IRQ
   if (INTCONbits.TMR0IF == 1)
   {
	  if (flags.rec)
	  {
	  	wpointer++;
	  	wpointer&=0x3F;
		po_b=ADRESH;
		if (po_b==0) po_b=1;	//begrenzen auf +1
	  	buffer[wpointer]=po_b;
	  	ADCON0bits.GO = 1;  // Set the GO bit to start a conversion
	  }


	  if (flags.play)
	  {
	  	if (rpointer!=wpointer)
		{
			rpointer++;
	  		rpointer&=0x3F;
	  		PORTD=buffer[rpointer];
			TMR0L+=playspeed;
		}
	  }

    


	  INTCONbits.TMR0IF = 0;
   }
}



void init_main()
{
   PORTA = 0x00;
   PORTB = 0xFF; 
   PORTC = 0xFE;
   PORTD = 0x00;
   PORTE = 0x00;	

   //init ports
   TRISA = 0b11111111 ;
   TRISB = 0b11110000;
   TRISC = 0b10010000;
   TRISD = 0b00000000;
   TRISE = 0x00;    // be careful Port E is only 3 Bits and the higher Bits 







   //CMCON = 0x07 ; // Comparator abschalten


   //Timer0-Configurtion
   // Teiler: 1:1 8Bit Timer 5Mhz: berlauf bei 19.5kHz
   T0CON = 0b11001000;


   irq_init();

   //Timer1-Configurtion
   //00 = 1:1 Prescale value 1
   //timer1 increment = 1/(FOSC/4) by 10MHZ => 1/(10MHz/4) = 400ns
   //By 16-bit-Timer1-Register => overflow 0xFFFF * 400ns = 26,2ms
   T1CON = 0b10000001;

   //ADC-Konfiguration
   //A/D Conversion Clock = FOSC/2 = 5MHz
   //Analog Channel Select, default AN0
   //10bits Format 0b000000xxxxxxxxxx

   ADCON0 = 0b10000001;
   // All Analog Pins are configured as Input although RA5 is digital Output
   //ADCON1 = 0b10000010;
   ADCON1=0b00000111;

   //adc_disable();


   // init rsr232
   TXSTA=0x22 ;
   RCSTA=0x90 ;
   SPBRG=64 ;   // 19k2

    //init PWM out
	T2CON=0b00000101; //Prescaler 4
	PR2=0xFF;
	//CCP1CON=0b00001100; // PWM Mode	

   irq_init();
}

unsigned char getc(void)
{
	return RCREG;
}


unsigned char kbhit(void)
{
char dummyb;

 if (RCSTAbits.OERR || RCSTAbits.FERR )
 {
   dummyb=RCREG;
   RCSTAbits.CREN=0;
   RCSTAbits.CREN=1;
   return 0;
 }


 if (PIR1bits.RCIF==1)
    return 1;
 else
    return 0;
}


void beep()
{
unsigned char beep_b;
unsigned char beep_i;

	Delay_ms(50); 
	ENP=1; //enable Audio out
	for (beep_i=0;beep_i<255;beep_i++)
		for (beep_b=0;beep_b<150;beep_b+=2)
		{
		PORTD=beep_b;
		}
	ENP=0; //disable Audio out
	Delay_ms(100); 
}


void aufnahme(void)
{
unsigned int i;				// Variable 0...65535

   ADCON0 = 0b10000001;		//Select Microphone
   ADCON1 = 0b00000100;

    beep();
	Delay_ms(100); 

	wpointer=0;
	rpointer=0;

	flags.rec=1;

	P_LED2=1;

	adl=0;
	adh=taste & 0x7F;
	adh=adh*4;

	TRISB=0xF0;
	PORTB=0x00;
	Delay10TCY();
	Delay_ms(100); 

	for (block=0;((block<511)&&((PORTB & 0xF0) != 0xF0));block++)	//512=13sec, Abbruck bei Taste losgelassen
	{

		if (Command(0x58,adh,adl,0xFF) !=0)
			 putc('w');

		SPI(0xFF);
		SPI(0xFF);
		SPI(0xFE);
	
		for(i=0; i < 512; i++)
		{
			while(rpointer==wpointer);
			rpointer++;
			rpointer&=0x3F;
			SPI(buffer[rpointer]);
		}
		SPI(255);				// Am Ende 2 Byte's ohne Bedeutung senden
		SPI(255);
		i=SPI(0xFF);
		i &=0b00011111;
		if (i != 0b00000101) putc('?');
		while(SPI(0xFF) !=0xFF);		// warten auf Ende des Busy -Zustandes.

		adl+=512;
		if (adl==0)	adh++;
	}

	flags.rec=0;
	P_LED2=0;


		if (Command(0x58,adh,adl,0xFF) !=0)
			 putc('w');

		SPI(0xFF);
		SPI(0xFF);
		SPI(0xFE);
	
		for(i=0; i < 512; i++)
		{
			SPI(0);	//EndeMarkierung setzen
		}
		SPI(255);				// Am Ende 2 Byte's ohne Bedeutung senden
		SPI(255);
		i=SPI(0xFF);
		i &=0b00011111;
		if (i != 0b00000101) putc('?');
		while(SPI(0xFF) !=0xFF);		// warten auf Ende des Busy -Zustandes.




    beep();beep();


}


void wiedergabe(void)
{
unsigned int i;				// Variable 0...65535
unsigned char w_b;

    ADCON0 = 0b10001001;		//Select Wiedergabegeschwindigkeit;
    ADCON1 = 0b00000100;

	ADCON0bits.GO = 1;
	while(ADCON0bits.GO);
	w_b=ADRESH;
	ADCON0bits.GO = 1;
	while(ADCON0bits.GO);
	w_b=ADRESH;
	if (w_b>192) w_b=192;
	playspeed=w_b;
	



	ENP=1; //enable Audio out
	wpointer=0;
	rpointer=0;
	flags.play=1;

	adl=0;
	adh=taste & 0x7F;
	adh=adh*4;

	for (block=0;((block<511) & (flags.play==1));block++)	//512=13sec Abbruch bei Endemarkierung
	{


		//******************************************************************************
		// Lesen  512 Byte-Mode 
		if (Command(0x51,adh,adl,0xFF) !=0)
			putc('?');
        while(SPI(0xFF) != 0xFE);			// Warten auf 0xFE, Anfang jeder Datenbertr.
		for(i=0; i < 512; i++)
		{
			//while(!PIR1bits.TXIF);   			// Prfen ob Register TXREG leer ist
			//TXREG =SPI(0xFF);			// Datenbyte senden
			w_b=SPI(0xFF);
			if (w_b==0) flags.play=0;

			if (flags.play)
			{
				while(rpointer==((wpointer+1)&0x3F));
				wpointer++;
				wpointer&=0x3F;
				buffer[wpointer]=w_b;
			}
		}
		SPI(0xFF);				// Am Ende 2 Byte's ohne Bedeutung
		SPI(0xFF);
		//************************************************************************
		adl+=512;
		if (adl==0)	adh++;
	}

	flags.play=0;
	ENP=0; //disable Audio out

}

unsigned char tast()
{
unsigned char ti;
unsigned char tastr;

    ADCON1=0b00000111;

	tastr=0;
	taste=0;
	PORTB=0b11111110;
	Delay10TCY();
	ti=PORTB;
	if ((ti & 0x10)==0){taste=0;tastr=1;}
	if ((ti & 0x20)==0){taste=1;tastr=1;}
	if ((ti & 0x40)==0){taste=2;tastr=1;}
	if ((ti & 0x80)==0){taste=3;tastr=1;}

	PORTB=0b11111101;
	Delay10TCY();
	ti=PORTB;
	if ((ti & 0x10)==0){taste=4;tastr=1;}
	if ((ti & 0x20)==0){taste=5;tastr=1;}
	if ((ti & 0x40)==0){taste=6;tastr=1;}
	if ((ti & 0x80)==0){taste=7;tastr=1;}

	PORTB=0b11111011;
	Delay10TCY();
	ti=PORTB;
	if ((ti & 0x10)==0){taste=8;tastr=1;}
	if ((ti & 0x20)==0){taste=9;tastr=1;}
	if ((ti & 0x40)==0){taste=10;tastr=1;}
	if ((ti & 0x80)==0){taste=11;tastr=1;}

	PORTB=0b11110111;
	Delay10TCY();
	ti=PORTB;
	if ((ti & 0x10)==0){taste=12;tastr=1;}
	if ((ti & 0x20)==0){taste=13;tastr=1;}
	if ((ti & 0x40)==0){taste=14;tastr=1;}
	if ((ti & 0x80)==0){taste=15;tastr=1;}

	ti=PORTA << 2;
	ti&=0xF0;
	taste|=ti;

	if (tastr) 	Delay_ms(20); 
	return tastr;
}

void main (void)
{

    irq_disable_global();
	P_LED2=0;

    init_main();
	flags.rec=0;
	flags.play=0;

    irq_enable_global();

    beep(); beep();


	

	while(1)
	{
		
		PORTD=0;
		PORTB=0;
		SSPCON1bits.SSPEN=0;	// SPI Ein
		PORTC=0x00;
		ENP=0;
		VSW=1; //disable VSW
		INTCONbits.TMR0IE=0;
		INTCONbits.RBIE=1;
		Sleep();
		INTCONbits.RBIE=0;
		INTCONbits.TMR0IE=1;

		VSW=0; //enable VSW
		Delay_ms(10); 
		
		if (tast()!=0)
		{
			VSW=0; //enable VSW
			Delay_ms(100); 

					
			MMC_Init();

			if (taste & 0x80) 
				aufnahme();
			else
				wiedergabe();
			VSW=1; //disable VSW

			while(tast());	//warten bis losgelassen
		}

	}



}

